@@ -76,6 +76,7 @@ struct GSListenerPrivate
         DBusConnection *system_connection;
 
         guint           active : 1;
+        guint           lid_closed : 1;
         guint           blanked : 1;
         time_t          blanked_start;
         char           *session_id;
@@ -84,6 +85,7 @@ struct GSListenerPrivate
 #ifdef WITH_SYSTEMD
         gboolean        have_systemd;
         char           *sd_session_id;
+        int             delay_fd;
 #endif
 
         dbus_uint32_t   inhibit_last_cookie;
@@ -107,6 +109,7 @@ enum {
 enum {
         PROP_0,
         PROP_ACTIVE,
+        PROP_LID_CLOSED,
 };
 
 static DBusObjectPathVTable
@@ -121,12 +124,20 @@ static guint         signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GSListener, gs_listener, G_TYPE_OBJECT)
 
+gboolean
+gs_listener_is_lid_closed (GSListener *listener)
+{
+        return listener->priv->lid_closed;
+}
+
 void
 gs_listener_send_switch_greeter (GSListener *listener)
 {
         dbus_bool_t sent;
         DBusMessage *message;
 
+        gs_debug ("Send switch greeter");
+
 #ifdef WITH_SYSTEMD
         /* Compare with 0. On failure this will return < 0.
          * In the later case we probably aren't using systemd.
@@ -166,6 +177,8 @@ gs_listener_send_lock_session (GSListener *listener)
         dbus_bool_t sent;
         DBusMessage *message;
 
+        gs_debug ("Send lock session");
+
 #ifdef WITH_SYSTEMD
         /* Compare with 0. On failure this will return < 0.
          * In the later case we probably aren't using systemd.
@@ -342,6 +355,170 @@ gs_listener_set_active (GSListener *listener,
         return TRUE;
 }
 
+void
+gs_listener_set_idle_hint (GSListener *listener, gboolean idle)
+{
+        dbus_bool_t sent;
+        DBusMessage *message;
+
+        gs_debug ("Send idle hint: %d", idle);
+
+#ifdef WITH_SYSTEMD
+        if (listener->priv->have_systemd) {
+
+                if (listener->priv->system_connection == NULL) {
+                        gs_debug ("No connection to the system bus");
+                        return;
+                }
+
+                message = dbus_message_new_method_call (SYSTEMD_LOGIND_SERVICE,
+                                                        listener->priv->session_id,
+                                                        SYSTEMD_LOGIND_SESSION_INTERFACE,
+                                                        "SetIdleHint");
+                if (message == NULL) {
+                        gs_debug ("Couldn't allocate the dbus message");
+                        return;
+                }
+
+                if (dbus_message_append_args (message,
+                                              DBUS_TYPE_BOOLEAN, &idle,
+                                              DBUS_TYPE_INVALID) == FALSE) {
+                        gs_debug ("Couldn't add args to the dbus message");
+                        dbus_message_unref (message);
+                        return;
+                }
+
+                sent = dbus_connection_send (listener->priv->system_connection, message, NULL);
+                dbus_message_unref (message);
+
+                if (sent == FALSE) {
+                        gs_debug ("Couldn't send the dbus message");
+                        return;
+                }
+
+                return;
+        }
+#endif
+
+#ifdef WITH_CONSOLE_KIT
+        if (listener->priv->system_connection == NULL) {
+                gs_debug ("No connection to the system bus");
+                return;
+        }
+
+        message = dbus_message_new_method_call (CK_SERVICE,
+                                                listener->priv->session_id,
+                                                CK_SESSION_INTERFACE,
+                                                "SetIdleHint");
+        if (message == NULL) {
+                gs_debug ("Couldn't allocate the dbus message");
+                return;
+        }
+
+        if (dbus_message_append_args (message,
+                                      DBUS_TYPE_BOOLEAN, &idle,
+                                      DBUS_TYPE_INVALID) == FALSE) {
+                gs_debug ("Couldn't add args to the dbus message");
+                dbus_message_unref (message);
+                return;
+        }
+
+        sent = dbus_connection_send (listener->priv->system_connection, message, NULL);
+        dbus_message_unref (message);
+
+        if (sent == FALSE) {
+                gs_debug ("Couldn't send the dbus message");
+                return;
+        }
+#endif
+}
+
+void
+gs_listener_delay_suspend (GSListener *listener)
+{
+#ifdef WITH_SYSTEMD
+        DBusMessage    *message;
+        DBusMessage    *reply;
+        DBusError       error;
+        const char     *what;
+        const char     *who;
+        const char     *why;
+        const char     *mode;
+        int             fd;
+
+        gs_debug ("Delay suspend");
+
+        if (listener->priv->system_connection == NULL) {
+                gs_debug ("No connection to the system bus");
+                return;
+        }
+
+        dbus_error_init (&error);
+
+        message = dbus_message_new_method_call (SYSTEMD_LOGIND_SERVICE, SYSTEMD_LOGIND_PATH, SYSTEMD_LOGIND_INTERFACE, "Inhibit");
+        if (message == NULL) {
+                gs_debug ("Couldn't allocate the dbus message");
+                return;
+        }
+
+        what = "sleep";
+        who  = _("Screen Locker");
+        why  = _("Lock the screen on suspend/resume");
+        mode = "delay";
+
+        if (dbus_message_append_args (message,
+                                      DBUS_TYPE_STRING, &what,
+                                      DBUS_TYPE_STRING, &who,
+                                      DBUS_TYPE_STRING, &why,
+                                      DBUS_TYPE_STRING, &mode,
+                                      DBUS_TYPE_INVALID) == FALSE) {
+                gs_debug ("Couldn't add args to the dbus message");
+                dbus_message_unref (message);
+                return;
+        }
+
+        reply = dbus_connection_send_with_reply_and_block (listener->priv->system_connection,
+                                                           message,
+                                                           -1, &error);
+        dbus_message_unref (message);
+
+        if (dbus_error_is_set (&error)) {
+                gs_debug ("%s raised:\n %s\n\n", error.name, error.message);
+                dbus_error_free (&error);
+                return;
+        }
+
+        if (dbus_message_get_args (reply, &error,
+                                   DBUS_TYPE_UNIX_FD, &fd,
+                                   DBUS_TYPE_INVALID) == FALSE) {
+                fd = -1;
+        }
+
+        dbus_message_unref (reply);
+
+        if (dbus_error_is_set (&error)) {
+                gs_debug ("%s raised:\n %s\n\n", error.name, error.message);
+                dbus_error_free (&error);
+                return;
+        }
+
+        listener->priv->delay_fd = fd;
+#endif
+}
+
+void
+gs_listener_resume_suspend (GSListener *listener)
+{
+#ifdef WITH_SYSTEMD
+        gs_debug ("Resume suspend: fd=%d", listener->priv->delay_fd);
+
+        if (listener->priv->delay_fd >= 0) {
+                close (listener->priv->delay_fd);
+                listener->priv->delay_fd = -1;
+        }
+#endif
+}
+
 static dbus_uint32_t
 gs_listener_add_inhibit (GSListener *listener,
                          const char *owner)
@@ -989,7 +1166,76 @@ query_session_active (GSListener *listener)
 }
 #endif
 
-#ifdef WITH_SYSTEMD
+#ifdef WITH_UPOWER
+#ifdef WITH_LOCK_ON_LID
+static gboolean
+query_lid_closed (GSListener *listener)
+{
+        DBusMessage    *message;
+        DBusMessage    *reply;
+        DBusError       error;
+        DBusMessageIter reply_iter;
+        DBusMessageIter sub_iter;
+        dbus_bool_t     active = FALSE;
+        const char     *interface;
+        const char     *property;
+
+        if (listener->priv->system_connection == NULL) {
+                gs_debug ("No connection to the system bus");
+                return FALSE;
+        }
+
+        dbus_error_init (&error);
+
+        message = dbus_message_new_method_call (UP_SERVICE, UP_PATH, DBUS_PROPERTIES_INTERFACE, "Get");
+        if (message == NULL) {
+                gs_debug ("Couldn't allocate the dbus message");
+                return FALSE;
+        }
+
+        interface = UP_INTERFACE;
+        property = "LidIsClosed";
+
+        if (dbus_message_append_args (message, DBUS_TYPE_STRING, &interface, DBUS_TYPE_STRING, &property, DBUS_TYPE_INVALID) == FALSE) {
+                gs_debug ("Couldn't add args to the dbus message");
+                dbus_message_unref (message);
+                return FALSE;
+        }
+
+        /* FIXME: use async? */
+        reply = dbus_connection_send_with_reply_and_block (listener->priv->system_connection,
+                                                           message,
+                                                           -1, &error);
+        dbus_message_unref (message);
+
+        if (dbus_error_is_set (&error)) {
+                gs_debug ("%s raised:\n %s\n\n", error.name, error.message);
+                dbus_error_free (&error);
+                return FALSE;
+        }
+
+        if (dbus_message_iter_init (reply, &reply_iter) == TRUE
+            && dbus_message_iter_get_arg_type (&reply_iter) == DBUS_TYPE_VARIANT) {
+
+                dbus_message_iter_recurse (&reply_iter, &sub_iter);
+
+                if (dbus_message_iter_get_arg_type (&sub_iter) == DBUS_TYPE_BOOLEAN) {
+                        dbus_message_iter_get_basic (&sub_iter, &active);
+                } else {
+                        gs_debug ("Unexpected return type");
+                }
+        } else {
+                gs_debug ("Unexpected return type");
+        }
+
+        dbus_message_unref (reply);
+
+        return active;
+}
+#endif
+#endif
+
+#if defined(WITH_SYSTEMD) || (defined(WITH_UPOWER) && defined(WITH_LOCK_ON_LID))
 static gboolean
 properties_changed_match (DBusMessage *message,
                           const char  *property)
@@ -1112,9 +1358,20 @@ listener_dbus_handle_system_message (DBusConnection *connection,
 
                                 /* Do a DBus query, since the sd_session_is_active isn't up to date. */
                                 new_active = query_session_active (listener);
+                                gs_debug ("systemd notified ActiveSession %d", new_active);
                                 g_signal_emit (listener, signals [SESSION_SWITCHED], 0, new_active);
                         }
 
+#ifdef WITH_UPOWER
+#ifdef WITH_LOCK_ON_LID
+                        if (properties_changed_match (message, "LidIsClosed")) {
+                                listener->priv->lid_closed = query_lid_closed (listener);
+                                gs_debug ("UPower notified LidIsClosed %d", (int)listener->priv->lid_closed);
+                                g_object_notify (G_OBJECT (listener), "lid-closed");
+                        }
+#endif
+#endif
+
                         return DBUS_HANDLER_RESULT_HANDLED;
                 }
 
@@ -1206,6 +1463,18 @@ listener_dbus_handle_system_message (DBusConnection *connection,
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
 #endif
+#ifdef WITH_LOCK_ON_LID
+        if (dbus_message_is_signal (message, DBUS_INTERFACE_PROPERTIES, "PropertiesChanged")) {
+
+                if (properties_changed_match (message, "LidIsClosed")) {
+                        listener->priv->lid_closed = query_lid_closed (listener);
+                        gs_debug ("UPower notified LidIsClosed %d", (int)listener->priv->lid_closed);
+                        g_object_notify (G_OBJECT (listener), "lid-closed");
+                }
+
+                return DBUS_HANDLER_RESULT_HANDLED;
+        }
+#endif
 #endif
 
        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
@@ -1389,6 +1658,9 @@ gs_listener_get_property (GObject            *object,
         case PROP_ACTIVE:
                 g_value_set_boolean (value, self->priv->active);
                 break;
+        case PROP_LID_CLOSED:
+                g_value_set_boolean (value, self->priv->lid_closed);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -1516,6 +1788,13 @@ gs_listener_class_init (GSListenerClass *klass)
                                                                NULL,
                                                                FALSE,
                                                                G_PARAM_READWRITE));
+        g_object_class_install_property (object_class,
+                                         PROP_LID_CLOSED,
+                                         g_param_spec_boolean ("lid-closed",
+                                                               NULL,
+                                                               NULL,
+                                                               FALSE,
+                                                               G_PARAM_READABLE));
 
         g_type_class_add_private (klass, sizeof (GSListenerPrivate));
 }
@@ -1701,6 +1980,17 @@ gs_listener_acquire (GSListener *listener,
                                             NULL);
 #endif
 
+#ifdef WITH_UPOWER
+#ifdef WITH_LOCK_ON_LID
+                        dbus_bus_add_match (listener->priv->system_connection,
+                                            "type='signal'"
+                                            ",sender='"UP_SERVICE"'"
+                                            ",interface='"DBUS_INTERFACE_PROPERTIES"'"
+                                            ",member='PropertiesChanged'",
+                                            NULL);
+#endif
+#endif
+
                         return (res != -1);
                 }
 #endif
@@ -1741,6 +2031,14 @@ gs_listener_acquire (GSListener *listener,
                                     ",member='Resuming'",
                                     NULL);
 #endif
+#ifdef WITH_LOCK_ON_LID
+                dbus_bus_add_match (listener->priv->system_connection,
+                                    "type='signal'"
+                                    ",sender='"UP_SERVICE"'"
+                                    ",interface='"DBUS_INTERFACE_PROPERTIES"'"
+                                    ",member='PropertiesChanged'",
+                                    NULL);
+#endif
 #endif
         }
 
@@ -1792,7 +2090,7 @@ query_session_id (GSListener *listener)
                         return NULL;
                 }
 
-                if (dbus_message_get_args (reply, &error, 
+                if (dbus_message_get_args (reply, &error,
                                            DBUS_TYPE_OBJECT_PATH, &ssid,
                                            DBUS_TYPE_INVALID)) {
                         ssid = g_strdup (ssid);
@@ -1831,7 +2129,7 @@ query_session_id (GSListener *listener)
                 return NULL;
         }
 
-        if (dbus_message_get_args (reply, &error, 
+        if (dbus_message_get_args (reply, &error,
                                    DBUS_TYPE_OBJECT_PATH, &ssid,
                                    DBUS_TYPE_INVALID)) {
                 ssid = g_strdup (ssid);
@@ -1972,6 +2270,7 @@ gs_listener_init (GSListener *listener)
 #ifdef WITH_SYSTEMD
         /* check if logind is running */
         listener->priv->have_systemd = (access("/run/systemd/seats/", F_OK) >= 0);
+        listener->priv->delay_fd = -1;
 #endif
 
         gs_listener_dbus_init (listener);
