@@ -83,6 +83,13 @@ gboolean    gs_listener_set_active              (GSListener *listener,
                                                  gboolean    active);
 void        gs_listener_send_switch_greeter     (GSListener *listener);
 void        gs_listener_send_lock_session       (GSListener *listener);
+gboolean    gs_listener_is_lid_closed           (GSListener *listener);
+
+void        gs_listener_delay_suspend           (GSListener *listener);
+void        gs_listener_resume_suspend          (GSListener *listener);
+
+void        gs_listener_set_idle_hint           (GSListener *listener,
+                                                 gboolean    idle);
 
 G_END_DECLS
 
