@@ -49,10 +49,9 @@ struct GSManagerPrivate
         guint        active : 1;
         guint        visible : 1;
         guint        blank : 1;
+        guint        closed : 1;
         guint        show_content : 1;
 
-        time_t       activate_time;
-
         guint        greeter_timeout_id;
         guint        lock_timeout_id;
 
@@ -233,49 +232,14 @@ window_map_event_cb (GSWindow  *window,
 {
         gs_debug ("Handling window map_event event");
 
+        /* FIXME: only emit signal once */
+        g_signal_emit (manager, signals [ACTIVATED], 0);
+
         manager_maybe_grab_window (manager, window);
 
         return FALSE;
 }
 
-static void
-window_map_cb (GSWindow  *window,
-               GSManager *manager)
-{
-        gs_debug ("Handling window map event");
-}
-
-static void
-window_unmap_cb (GSWindow  *window,
-                 GSManager *manager)
-{
-        gs_debug ("window unmapped!");
-}
-
-static void
-manager_show_window (GSManager *manager,
-                     GSWindow  *window)
-{
-        manager->priv->activate_time = time (NULL);
-
-        /* FIXME: only emit signal once */
-        g_signal_emit (manager, signals [ACTIVATED], 0);
-}
-
-static void
-window_show_cb (GSWindow  *window,
-                GSManager *manager)
-{
-
-        g_return_if_fail (manager != NULL);
-        g_return_if_fail (GS_IS_MANAGER (manager));
-        g_return_if_fail (window != NULL);
-        g_return_if_fail (GS_IS_WINDOW (window));
-
-        gs_debug ("Handling window show");
-        manager_show_window (manager, window);
-}
-
 static void
 content_draw_cb (GtkWidget *widget,
                  cairo_t   *cr,
@@ -312,10 +276,7 @@ disconnect_window_signals (GSManager *manager,
         g_signal_handlers_disconnect_by_func (drawing_area, content_expose_cb, manager);
 #endif
 
-        g_signal_handlers_disconnect_by_func (window, window_show_cb, manager);
-        g_signal_handlers_disconnect_by_func (window, window_map_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_map_event_cb, manager);
-        g_signal_handlers_disconnect_by_func (window, window_unmap_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_grab_broken_cb, manager);
 }
 
@@ -343,14 +304,8 @@ connect_window_signals (GSManager *manager,
 
         g_signal_connect_object (window, "destroy",
                                  G_CALLBACK (window_destroyed_cb), manager, 0);
-        g_signal_connect_object (window, "show",
-                                 G_CALLBACK (window_show_cb), manager, G_CONNECT_AFTER);
-        g_signal_connect_object (window, "map",
-                                 G_CALLBACK (window_map_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "map_event",
                                  G_CALLBACK (window_map_event_cb), manager, G_CONNECT_AFTER);
-        g_signal_connect_object (window, "unmap",
-                                 G_CALLBACK (window_unmap_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "grab_broken_event",
                                  G_CALLBACK (window_grab_broken_cb), manager, G_CONNECT_AFTER);
 }
@@ -424,6 +379,15 @@ on_screen_monitors_changed (GdkScreen *screen,
                 gdk_flush ();
                 gdk_x11_ungrab_server ();
         }
+
+	for (l = manager->priv->windows; l != NULL; l = l->next) {
+		GdkScreen *this_screen;
+
+		this_screen = gs_window_get_screen (GS_WINDOW (l->data));
+		if (this_screen == screen) {
+			gtk_widget_queue_resize (GTK_WIDGET (l->data));
+		}
+	}
 }
 
 static void
@@ -556,7 +520,6 @@ gs_manager_finalize (GObject *object)
         gs_manager_destroy_windows (manager);
 
         manager->priv->active = FALSE;
-        manager->priv->activate_time = 0;
 
         gs_manager_stop_switch (manager);
         gs_manager_stop_lock (manager);
@@ -663,7 +626,7 @@ gs_manager_activate (GSManager *manager)
 
         show_windows (manager->priv->windows);
 
-        if (manager->priv->visible && !manager->priv->blank) {
+        if (manager->priv->visible && !manager->priv->blank && !manager->priv->closed) {
                 gs_manager_timed_switch (manager);
         }
 
@@ -695,7 +658,6 @@ gs_manager_deactivate (GSManager *manager)
 
         /* reset state */
         manager->priv->active = FALSE;
-        manager->priv->activate_time = 0;
         manager->priv->show_content = FALSE;
 
         return TRUE;
@@ -731,7 +693,7 @@ gs_manager_set_session_visible (GSManager *manager,
 {
         manager->priv->visible = visible;
 
-        if (manager->priv->active && visible && !manager->priv->blank) {
+        if (manager->priv->active && visible && !manager->priv->blank && !manager->priv->closed) {
                 gs_manager_timed_switch (manager);
         } else {
                 gs_manager_stop_switch (manager);
@@ -757,7 +719,7 @@ gs_manager_set_blank_screen (GSManager *manager,
                 gs_manager_timed_lock (manager);
         } else {
                 gs_manager_stop_lock (manager);
-                if (manager->priv->active && manager->priv->visible) {
+                if (manager->priv->active && manager->priv->visible && !manager->priv->closed) {
                         gs_manager_timed_switch (manager);
                 }
         }
@@ -772,6 +734,19 @@ gs_manager_get_blank_screen (GSManager *manager)
         return manager->priv->blank;
 }
 
+void
+gs_manager_set_lid_closed (GSManager *manager,
+                           gboolean   closed)
+{
+        manager->priv->closed = closed;
+
+        if (manager->priv->active && manager->priv->visible && !manager->priv->blank && !closed) {
+                gs_manager_timed_switch (manager);
+        } else {
+                gs_manager_stop_switch (manager);
+        }
+}
+
 void
 gs_manager_set_lock_after (GSManager *manager,
                            guint      lock_after)
